import java.util.Set;

public interface Graph {

    /**
     * Ein einfaches Interface fuer Graphen G = (V,E)
     */

    /**
     * @return die Anzahl der Knoten im Graphen
     */
    public int size();

    /**
     * @param  node ein Knoten
     * @return wahr falls node in V
     */
    public boolean containsNode(int node);

    /**
     * @param  source ein Knoten
     * @param  target ein Knoten
     * @return wahr falls (source,target) in E
     */
    public boolean containsEdge(int source, int target);

    /**
     * @return einen frischen Knoten
     */
    public int freshNode();

    /**
     * @param  source ein Knoten
     * @param  target ein Knoten
     * @return fuegt die Kante (source,target) in den Graphen ein
     */
    public void addEdge(int source, int target) throws NodeNotFoundException;

    /**
     * loescht die Kante (source,target) aus dem Graphen falls diese existiert
     * @param  source ein Knoten
     * @param  target ein Knoten
     */
    public void removeEdge(int source, int target);

    /**
     * loescht den Knoten node aus dem Graphen falls dieser existiert
     * @param  node ein Knoten
     */
    public void removeNode(int node);

    /**
     * @param  node ein Knoten
     * @return die Menge der Vorgaenger des gegebenen Knotens falls
     *         dieser vorhanden, ansonsten ein leeres Set
     */
    public Set<Integer> successors(int node);

    /**
     * @param  node ein Knoten
     * @return die Menge der Nachfolger des gegebenen Knotens falls
     *         dieser vorhanden, ansonsten ein leeres Set
     */
    public Set<Integer> predecessors(int node);

    /**
     * @param  node ein Knoten
     * @return Eingangsgrad des gegebenen Knoten, ansonsten 0
     */
    public int inDegree(int node);

    /**
     * @param  node ein Knoten
     * @return Ausgangsgrad des gegebenen Knoten, ansonsten 0
     */
    public int outDegree(int node);
}

